// Copyright 2018 Open Source Robotics Foundation, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef MEMORY_TOOLS__DISPATCH_CALLBACK_HPP_
#define MEMORY_TOOLS__DISPATCH_CALLBACK_HPP_

#include <atomic>

#include "osrf_testing_tools_cpp/memory_tools/register_hooks.hpp"

namespace osrf_testing_tools_cpp
{
namespace memory_tools
{

inline
void
dispatch_callback(
  const AnyMemoryToolsCallback * user_callback,
  MemoryToolsService & service)
{
  if (nullptr == user_callback) {
    return;
  }
  auto callback = user_callback;
  if (std::holds_alternative<std::nullptr_t>(*callback)) {
    return;
  }
  auto callback_ptr = std::get_if<MemoryToolsCallback>(callback);
  if (callback_ptr) {
    (*callback_ptr)(service);
  }
  auto simple_callback_ptr = std::get_if<MemoryToolsSimpleCallback>(callback);
  if (simple_callback_ptr) {
    (*simple_callback_ptr)();
  }
}

}  // namespace memory_tools
}  // namespace osrf_testing_tools_cpp

#endif  // MEMORY_TOOLS__DISPATCH_CALLBACK_HPP_
